import customtkinter as ctk
import requests
from PIL import Image, ImageTk
from tkinter import messagebox, filedialog
# import io

SHOPPING = "#00c1ff"
DARK_BG = "#1f1f1f"
CARD_BG = "#2b2b2b"

def create_shop_frame(parent, switch_func, API_URL, token):
    """
    Create shop page with a modern dark theme and improved layout.
    """
    # Main container frame
    frame = ctk.CTkFrame(parent, fg_color=DARK_BG)

    def update_token(new_token):
        nonlocal token
        token = new_token

    frame.update_token = update_token  # Add the method to the frame

    # --- TOP BAR ---
    top_bar = ctk.CTkFrame(frame, fg_color=SHOPPING, height=50)
    top_bar.pack(fill="x", side="top")

    top_label = ctk.CTkLabel(
        top_bar, 
        text="Create Shop", 
        text_color="white", 
        font=("Helvetica", 20, "bold")
    )
    top_label.pack(side="left", padx=25)

    def go_back():
        switch_func("dashboard")

    back_button = ctk.CTkButton(
        top_bar,
        text="← Back",
        fg_color="transparent",
        text_color="white",
        hover_color="#0096ff",
        command=go_back,
        width=80,
        height=35,
        font=("Helvetica", 14)
    )
    back_button.pack(side="right", padx=20, pady=7)

    # --- MAIN SECTION: Sidebar + Content ---
    main_section = ctk.CTkFrame(frame, fg_color="transparent")
    main_section.pack(fill="both", expand=True)

    # LEFT SIDEBAR
    sidebar_frame = ctk.CTkFrame(main_section, width=250, fg_color=CARD_BG, corner_radius=15)
    sidebar_frame.pack(side="left", fill="y", padx=20, pady=20)
    sidebar_frame.pack_propagate(False)

    sidebar_title = ctk.CTkLabel(
        sidebar_frame, 
        text="Menu", 
        font=("Helvetica", 18, "bold"), 
        text_color=SHOPPING
    )
    sidebar_title.pack(pady=(20, 10))

    def create_nav_button(text, command=None, is_active=False):
        return ctk.CTkButton(
            sidebar_frame,
            text=text,
            fg_color="transparent",
            text_color="white",
            hover_color="#3b3b3b",
            command=command,
            height=40,
            font=("Helvetica", 14),
            state="disabled" if is_active else "normal"
        )

    nav_dashboard = create_nav_button("Dashboard", go_back)
    nav_dashboard.pack(fill="x", padx=15, pady=5)

    def open_profile():
        switch_func("user_details")

    nav_profile = create_nav_button("My Profile", open_profile)
    nav_profile.pack(fill="x", padx=15, pady=5)

    nav_orders = create_nav_button("My Orders", lambda: switch_func("user_orders"))
    nav_orders.pack(fill="x", padx=15, pady=5)

    become_owner = create_nav_button("Become Shop Owner", is_active=True)
    become_owner.pack(fill="x", padx=15, pady=5)

    # RIGHT CONTENT (Create Shop Form)
    content_frame = ctk.CTkFrame(main_section, fg_color=CARD_BG, corner_radius=15)
    content_frame.pack(side="left", fill="both", expand=True, padx=(0, 20), pady=20)

    # Header
    header_frame = ctk.CTkFrame(content_frame, fg_color="transparent")
    header_frame.pack(fill="x", padx=30, pady=20)

    title_label = ctk.CTkLabel(
        header_frame,
        text="Create Your Shop",
        font=("Helvetica", 24, "bold"),
        text_color=SHOPPING
    )
    title_label.pack(anchor="w")

    subtitle_label = ctk.CTkLabel(
        header_frame,
        text="Fill in your shop details to get started",
        font=("Helvetica", 14),
        text_color="gray"
    )
    subtitle_label.pack(anchor="w", pady=(5, 0))

    # Main Content
    main_content = ctk.CTkFrame(content_frame, fg_color="transparent")
    main_content.pack(fill="both", expand=True, padx=30, pady=(0, 20))

    # LEFT FORM
    form_frame = ctk.CTkFrame(main_content, fg_color="transparent")
    form_frame.pack(side="left", fill="both", expand=True, padx=(0, 20))

    def create_form_field(label_text, placeholder_text):
        field_frame = ctk.CTkFrame(form_frame, fg_color="transparent")
        field_frame.pack(fill="x", pady=(0, 15))

        label = ctk.CTkLabel(
            field_frame,
            text=label_text,
            font=("Helvetica", 14),
            text_color="white"
        )
        label.pack(anchor="w")

        entry = ctk.CTkEntry(
            field_frame,
            height=40,
            corner_radius=8,
            placeholder_text=placeholder_text,
            fg_color="#3b3b3b",
            border_color=SHOPPING,
            text_color="white"
        )
        entry.pack(fill="x", pady=(5, 0))
        return entry

    shop_name_entry = create_form_field("Shop Name", "Enter your shop name...")
    shop_description_entry = create_form_field("Shop Description", "Enter your shop description...")
    shop_address_entry = create_form_field("Shop Address", "Enter your shop address...")

    # RIGHT PICTURE SECTION
    pic_frame = ctk.CTkFrame(main_content, fg_color="transparent")
    pic_frame.pack(side="left", fill="both", expand=True)

    pic_label = ctk.CTkLabel(
        pic_frame,
        text="Shop Logo",
        font=("Helvetica", 18, "bold"),
        text_color=SHOPPING
    )
    pic_label.pack(anchor="w", pady=(0, 10))

    photo_frame = ctk.CTkFrame(pic_frame, fg_color="#3b3b3b", corner_radius=10)
    photo_frame.pack(fill="x", pady=(0, 20))

    photo_label = ctk.CTkLabel(
        photo_frame,
        text="No image",
        text_color="gray",
        font=("Helvetica", 14)
    )
    photo_label.pack(pady=20)

    def choose_photo():
        file_path = filedialog.askopenfilename(
            title="Choose a Shop Logo",
            filetypes=[("Image Files", "*.png *.jpg *.jpeg *.gif")],
        )
        if file_path:
            try:
                pil_img = Image.open(file_path).resize((150, 150))
                tk_img = ImageTk.PhotoImage(pil_img)
                photo_label.configure(image=tk_img, text="")
                photo_label.image = tk_img
            except Exception as e:
                print(f"Image load error: {e}")
                messagebox.showerror("Error", "Cannot load the image.")

    change_photo_button = ctk.CTkButton(
        pic_frame,
        text="Choose Image",
        command=choose_photo,
        corner_radius=8,
        height=40,
        font=("Helvetica", 14),
        fg_color=SHOPPING,
        hover_color="#0096ff",
    )
    change_photo_button.pack(fill="x")

    # Create Shop Button
    def create_shop():
        shop_name = shop_name_entry.get().strip()
        shop_description = shop_description_entry.get().strip()
        shop_address = shop_address_entry.get().strip()

        if not all([shop_name, shop_description, shop_address]):
            messagebox.showwarning("Input Error", "All fields are required!")
            return

        headers = {"Authorization": f"Bearer {token}"}
        
        # Create form data
        form_data = {
            "name": (None, shop_name),
            "description": (None, shop_description),
            "address": (None, shop_address),
        }

        try:
            resp = requests.post(
                f"{API_URL}/shops/create", 
                headers=headers,
                files=form_data  # Use files parameter for form data
            )
            if resp.status_code == 200:
                messagebox.showinfo("Success", "Shop created successfully!")
                switch_func("dashboard")
            else:
                messagebox.showerror("Error", "Failed to create shop.")
        except Exception as e:
            messagebox.showerror("Error", f"Request error: {e}")

    create_button = ctk.CTkButton(
        form_frame,
        text="Create Shop",
        command=create_shop,
        corner_radius=8,
        height=45,
        font=("Helvetica", 14, "bold"),
        fg_color=SHOPPING,
        hover_color="#0096ff",
    )
    create_button.pack(fill="x", pady=(20, 0))

    return frame
