import customtkinter as ctk
import requests
from tkinter import messagebox
from datetime import datetime
from utils.api_requests import add_payment_method

SHOPPING = "#00c1ff"
DARK_BG = "#1f1f1f"
CARD_BG = "#2b2b2b"


def user_payments_frame(parent, switch_func, API_URL, token):
    """
    User payments page with a modern dark theme and improved layout.
    """
    # Main container frame
    frame = ctk.CTkFrame(parent, fg_color=DARK_BG)
    frame.token = token  # Store token as an attribute

    def update_token(new_token):
        """Update the token when it changes"""
        frame.token = new_token

    # Add the update_token method to the frame
    frame.update_token = update_token

    # --- TOP BAR ---
    top_bar = ctk.CTkFrame(frame, fg_color=SHOPPING, height=50)
    top_bar.pack(fill="x", side="top")

    top_label = ctk.CTkLabel(
        top_bar, text="My Payments", text_color="white", font=("Helvetica", 20, "bold")
    )
    top_label.pack(side="left", padx=25)

    def go_back():
        switch_func("dashboard")

    back_button = ctk.CTkButton(
        top_bar,
        text="← Back",
        fg_color="transparent",
        text_color="white",
        hover_color="#0096ff",
        command=go_back,
        width=80,
        height=35,
        font=("Helvetica", 14),
    )
    back_button.pack(side="right", padx=20, pady=7)

    # --- MAIN SECTION: Sidebar + Content ---
    main_section = ctk.CTkFrame(frame, fg_color="transparent")
    main_section.pack(fill="both", expand=True)

    # LEFT SIDEBAR
    sidebar_frame = ctk.CTkFrame(
        main_section, width=250, fg_color=CARD_BG, corner_radius=15
    )
    sidebar_frame.pack(side="left", fill="y", padx=20, pady=20)
    sidebar_frame.pack_propagate(False)

    sidebar_title = ctk.CTkLabel(
        sidebar_frame, text="Menu", font=("Helvetica", 18, "bold"), text_color=SHOPPING
    )
    sidebar_title.pack(pady=(20, 10))

    def create_nav_button(text, command=None, is_active=False):
        return ctk.CTkButton(
            sidebar_frame,
            text=text,
            fg_color="transparent",
            text_color="white",
            hover_color="#3b3b3b",
            command=command,
            height=40,
            font=("Helvetica", 14),
            state="disabled" if is_active else "normal",
        )

    nav_dashboard = create_nav_button("Dashboard", go_back)
    nav_dashboard.pack(fill="x", padx=15, pady=5)

    def open_profile():
        switch_func("user_details")

    nav_profile = create_nav_button("My Profile", open_profile)
    nav_profile.pack(fill="x", padx=15, pady=5)

    def open_orders():
        switch_func("user_orders")

    nav_orders = create_nav_button("My Orders", open_orders)
    nav_orders.pack(fill="x", padx=15, pady=5)

    nav_payments = create_nav_button("My Payments", is_active=True)
    nav_payments.pack(fill="x", padx=15, pady=5)

    # RIGHT CONTENT
    content_frame = ctk.CTkFrame(main_section, fg_color=CARD_BG, corner_radius=15)
    content_frame.pack(side="left", fill="both", expand=True, padx=(0, 20), pady=20)

    # Create a scrollable frame for the content
    content_scrollable = ctk.CTkScrollableFrame(
        content_frame,
        fg_color="transparent",
        scrollbar_button_color=SHOPPING,
        scrollbar_button_hover_color="#0096ff"
    )
    content_scrollable.pack(fill="both", expand=True)

    # Header
    header_frame = ctk.CTkFrame(content_scrollable, fg_color="transparent")
    header_frame.pack(fill="x", padx=30, pady=20)

    title_label = ctk.CTkLabel(
        header_frame,
        text="Payment Methods",
        font=("Helvetica", 24, "bold"),
        text_color=SHOPPING,
    )
    title_label.pack(anchor="w")

    subtitle_label = ctk.CTkLabel(
        header_frame,
        text="Add and manage your payment methods",
        font=("Helvetica", 14),
        text_color="gray",
    )
    subtitle_label.pack(anchor="w", pady=(5, 0))

    # Create two columns container
    columns_container = ctk.CTkFrame(content_scrollable, fg_color="transparent")
    columns_container.pack(fill="both", expand=True, padx=30, pady=(0, 20))
    columns_container.grid_columnconfigure(0, weight=1)
    columns_container.grid_columnconfigure(1, weight=1)

    # Left Column - Form and Preview
    left_column = ctk.CTkFrame(columns_container, fg_color="transparent")
    left_column.grid(row=0, column=0, sticky="nsew", padx=(0, 15))

    # Right Column - Current Payment Method
    right_column = ctk.CTkFrame(columns_container, fg_color="transparent")
    right_column.grid(row=0, column=1, sticky="nsew", padx=(15, 0))

    # Form Frame in Left Column
    form_frame = ctk.CTkFrame(left_column, fg_color="transparent")
    form_frame.pack(fill="x", pady=(0, 20))

    # Saved Payment Methods Section in Right Column
    saved_payments_frame = ctk.CTkFrame(right_column, fg_color="transparent")
    saved_payments_frame.pack(fill="x", pady=(0, 40))

    saved_payments_label = ctk.CTkLabel(
        saved_payments_frame,
        text="Current Payment Method",
        font=("Helvetica", 18, "bold"),
        text_color="white",
    )
    saved_payments_label.pack(pady=(0, 20))

    # Container for card and buttons with fixed width for proper centering
    saved_card_section = ctk.CTkFrame(saved_payments_frame, fg_color="transparent", width=600, height=220)
    saved_card_section.pack(expand=True)
    saved_card_section.pack_propagate(False)
    saved_card_section.grid_columnconfigure(0, weight=1)
    saved_card_section.grid_columnconfigure(1, weight=0)

    # Container for the saved payment card
    saved_card_container = ctk.CTkFrame(saved_card_section, fg_color="transparent")
    saved_card_container.grid(row=0, column=0, sticky="e", padx=(0, 20))

    # Create the saved card frame (initially empty)
    saved_card_frame = None

    # Buttons container
    buttons_container = ctk.CTkFrame(saved_card_section, fg_color="transparent")
    buttons_container.grid(row=0, column=1, sticky="w", pady=(60, 0))

    def create_payment_card(payment):
        """Create or update the saved payment card display"""
        nonlocal saved_card_frame
        
        # Clear buttons container first
        for widget in buttons_container.winfo_children():
            widget.destroy()
            
        # Remove existing card if any
        if saved_card_frame is not None:
            saved_card_frame.destroy()
            saved_card_frame = None

        # If no payment data, don't create a new card
        if not payment:
            return

        # Create new card frame
        saved_card_frame = ctk.CTkFrame(
            saved_card_container,
            corner_radius=15,
            height=200,
            width=350
        )
        saved_card_frame.pack(anchor="e")
        saved_card_frame.pack_propagate(False)

        # Set card color based on payment method
        card_type = payment['payment_method'].upper()
        if card_type == "VISA":
            saved_card_frame.configure(fg_color="#0066cc")
        elif card_type == "MASTERCARD":
            saved_card_frame.configure(fg_color="#EB001B")
        elif card_type == "PAYPAL":
            saved_card_frame.configure(fg_color="#003087")

        # Chip image
        chip_frame = ctk.CTkFrame(
            saved_card_frame,
            width=50,
            height=40,
            fg_color="#FFD700",
            corner_radius=5
        )
        chip_frame.place(x=30, y=50)

        # Card type label
        card_type_label = ctk.CTkLabel(
            saved_card_frame,
            text=card_type,
            font=("Helvetica", 20, "bold"),
            text_color="white"
        )
        card_type_label.place(x=270, y=20)

        # Format card number with spaces
        raw_number = payment['card_number']
        # Mask first 12 digits and show last 4
        masked_number = '**** **** **** ' + raw_number[-4:] if len(raw_number) >= 4 else raw_number
        formatted_number = masked_number

        # Card number
        card_number = ctk.CTkLabel(
            saved_card_frame,
            text=formatted_number,
            font=("Courier", 24, "bold"),
            text_color="white"
        )
        card_number.place(x=30, y=100)

        # Expiry date label
        expiry_label = ctk.CTkLabel(
            saved_card_frame,
            text="VALID\nTHRU",
            font=("Helvetica", 8),
            text_color="white"
        )
        expiry_label.place(x=30, y=140)

        # Expiry date
        expiry_date = ctk.CTkLabel(
            saved_card_frame,
            text=payment['expiry_date'],
            font=("Helvetica", 14, "bold"),
            text_color="white"
        )
        expiry_date.place(x=30, y=165)

        # Cardholder name
        cardholder_name = ctk.CTkLabel(
            saved_card_frame,
            text="CARDHOLDER NAME",
            font=("Helvetica", 14, "bold"),
            text_color="white"
        )
        cardholder_name.place(x=150, y=165)

        # Add buttons with improved styling
        edit_button = ctk.CTkButton(
            buttons_container,
            text="Edit",
            width=120,
            height=38,
            corner_radius=20,
            command=lambda: edit_payment(payment),
            fg_color="#00B2FF",
            hover_color="#0095D9",
            font=("Helvetica", 14, "bold"),
            border_width=2,
            border_color="#80D9FF"
        )
        edit_button.pack(pady=(0, 15))

        # Delete button with matching style
        delete_button = ctk.CTkButton(
            buttons_container,
            text="Delete",
            width=120,
            height=38,
            corner_radius=20,
            command=lambda: delete_payment(payment),
            fg_color="#FF4444",
            hover_color="#D93939",
            font=("Helvetica", 14, "bold"),
            border_width=2,
            border_color="#FFA4A4"
        )
        delete_button.pack()

    def disable_form_inputs():
        """Disable all form inputs"""
        card_entry.configure(state="disabled")
        month_entry.configure(state="disabled")
        year_entry.configure(state="disabled")
        cvv_entry.configure(state="disabled")
        card_type_dropdown.configure(state="disabled")
        save_button.configure(state="disabled")

    def enable_form_inputs():
        """Enable all form inputs"""
        card_entry.configure(state="normal")
        month_entry.configure(state="normal")
        year_entry.configure(state="normal")
        cvv_entry.configure(state="normal")
        card_type_dropdown.configure(state="normal")
        save_button.configure(state="normal")

    def clear_form():
        """Clear all form fields and reset to default state"""
        card_entry.delete(0, "end")
        month_entry.delete(0, "end")
        year_entry.delete(0, "end")
        cvv_entry.delete(0, "end")
        card_type_var.set("Visa")
        if hasattr(frame, 'current_payment_id'):
            delattr(frame, 'current_payment_id')
        update_card_preview()
        # Enable form inputs when clearing
        enable_form_inputs()

    def edit_payment(payment):
        """Populate form with payment data for editing"""
        nonlocal saved_card_frame
        
        try:
            # Enable form inputs for editing
            enable_form_inputs()
            
            # Store payment ID for update
            frame.current_payment_id = payment['id']
            
            # Populate form with existing payment data
            card_entry.delete(0, "end")
            card_entry.insert(0, payment['card_number'])
            
            # Set card type (capitalize first letter)
            card_type = payment['payment_method'].capitalize()
            card_type_var.set(card_type)
            
            # Set expiry date
            month, year = payment['expiry_date'].split('/')
            month_entry.delete(0, "end")
            month_entry.insert(0, month)
            year_entry.delete(0, "end")
            year_entry.insert(0, year)
            
            # Update both previews
            update_card_preview()
            
            # Create a temporary payment object for the saved card preview
            temp_payment = {
                'id': payment['id'],
                'payment_method': card_type,
                'card_number': payment['card_number'],
                'expiry_date': payment['expiry_date']
            }
            create_payment_card(temp_payment)
        except Exception as e:
            messagebox.showerror("Error", f"Failed to load payment data: {str(e)}")

    def delete_payment(payment):
        """Delete payment method and update UI"""
        nonlocal saved_card_frame
        
        if messagebox.askyesno("Confirm Delete", "Are you sure you want to delete this payment method?"):
            try:
                headers = {"Authorization": f"Bearer {frame.token}"}
                response = requests.delete(
                    f"{API_URL}/payment/{payment['id']}",
                    headers=headers
                )
                
                if response.status_code == 200:
                    # Clear UI elements
                    if saved_card_frame is not None:
                        saved_card_frame.destroy()
                        saved_card_frame = None
                    
                    # Clear buttons container
                    for widget in buttons_container.winfo_children():
                        widget.destroy()
                    
                    # Clear form and reset state
                    clear_form()
                    
                    # Remove the current_payment_id if it exists
                    if hasattr(frame, 'current_payment_id'):
                        delattr(frame, 'current_payment_id')
                    
                    messagebox.showinfo("Success", "Payment method deleted successfully")
                    
                    # Enable form inputs after successful delete
                    enable_form_inputs()
                    
                    # Refresh the UI
                    refresh_payments()
                else:
                    error_msg = response.json().get("detail", "Failed to delete payment method")
                    messagebox.showerror("Error", error_msg)
            except Exception as e:
                messagebox.showerror("Error", f"Failed to delete payment method: {str(e)}")

    def refresh_payments():
        """Fetch and display the current payment method"""
        nonlocal saved_card_frame
        
        try:
            headers = {"Authorization": f"Bearer {frame.token}"}
            response = requests.get(f"{API_URL}/payment/user", headers=headers)
            
            if response.status_code == 200:
                payments = response.json()
                # Clear existing card if any
                if saved_card_frame is not None:
                    saved_card_frame.destroy()
                    saved_card_frame = None
                
                # Clear buttons container
                for widget in buttons_container.winfo_children():
                    widget.destroy()
                
                if payments and len(payments) > 0:
                    # Display the first/current payment method
                    create_payment_card(payments[0])
                    # Disable form inputs when payment exists
                    disable_form_inputs()
                else:
                    # If no payment methods, clear everything and enable form
                    clear_form()
                    enable_form_inputs()
            elif response.status_code == 401:
                # Handle unauthorized error - likely token expired
                messagebox.showerror("Authentication Error", "Your session has expired. Please log in again.")
            elif response.status_code == 404:
                # Handle 404 gracefully - just means no payment methods yet
                clear_form()
                enable_form_inputs()
            else:
                # Handle other errors
                error_msg = response.json().get("detail", "Unknown error")
                messagebox.showerror("Error", f"Failed to fetch payment methods: {error_msg}")
        except Exception as e:
            # Enable the form anyway so user can add payment
            clear_form()
            enable_form_inputs()
            # Show a less alarming message
            print(f"Payment fetch error: {str(e)}")
            # Don't show error to user, just let them add a payment

    # Card Number Field
    card_label = ctk.CTkLabel(
        form_frame, text="Card Number", font=("Helvetica", 14), text_color="white"
    )
    card_label.pack(anchor="w", pady=(0, 5))

    card_entry = ctk.CTkEntry(
        form_frame,
        placeholder_text="Enter your card number",
        height=40,
        corner_radius=8,
    )
    card_entry.pack(fill="x", pady=(0, 15))

    # Card Type Selection
    card_type_label = ctk.CTkLabel(
        form_frame, text="Card Type", font=("Helvetica", 14), text_color="white"
    )
    card_type_label.pack(anchor="w", pady=(0, 5))

    card_type_var = ctk.StringVar(value="Visa")
    card_type_options = ["Visa", "Mastercard", "PayPal"]

    card_type_dropdown = ctk.CTkOptionMenu(
        form_frame,
        values=card_type_options,
        variable=card_type_var,
        width=200,
        height=40,
        corner_radius=8,
        fg_color="#3b3b3b",
        button_color=SHOPPING,
        button_hover_color="#0096ff",
        dropdown_fg_color="#3b3b3b",
        font=("Helvetica", 14),
    )
    card_type_dropdown.pack(anchor="w", pady=(0, 15))

    # Expiration Date
    exp_label = ctk.CTkLabel(
        form_frame,
        text="Expiration Date (MM/YY)",
        font=("Helvetica", 14),
        text_color="white",
    )
    exp_label.pack(anchor="w", pady=(0, 5))

    exp_frame = ctk.CTkFrame(form_frame, fg_color="transparent")
    exp_frame.pack(fill="x", pady=(0, 15))

    month_entry = ctk.CTkEntry(
        exp_frame, placeholder_text="MM", width=60, height=40, corner_radius=8
    )
    month_entry.pack(side="left", padx=(0, 5))

    slash_label = ctk.CTkLabel(
        exp_frame, text="/", font=("Helvetica", 14), text_color="white"
    )
    slash_label.pack(side="left", padx=5)

    year_entry = ctk.CTkEntry(
        exp_frame, placeholder_text="YY", width=60, height=40, corner_radius=8
    )
    year_entry.pack(side="left", padx=5)

    # Security Code
    cvv_label = ctk.CTkLabel(
        form_frame, text="Security Code", font=("Helvetica", 14), text_color="white"
    )
    cvv_label.pack(anchor="w", pady=(0, 5))

    cvv_entry = ctk.CTkEntry(
        form_frame,
        placeholder_text="CVV",
        width=100,
        height=40,
        corner_radius=8,
        show="*",
    )
    cvv_entry.pack(anchor="w")

    # Card Preview Section
    preview_frame = ctk.CTkFrame(form_frame, fg_color="transparent")
    preview_frame.pack(fill="x", pady=(20, 10))

    preview_label = ctk.CTkLabel(
        preview_frame,
        text="Card Preview",
        font=("Helvetica", 14),
        text_color="white"
    )
    preview_label.pack(anchor="center")

    # Card Preview Frame - Centered
    preview_container = ctk.CTkFrame(preview_frame, fg_color="transparent")
    preview_container.pack(fill="x", pady=(0, 20))

    card_preview = ctk.CTkFrame(
        preview_container,
        fg_color="#0066cc",
        corner_radius=15,
        height=200,
        width=350
    )
    card_preview.pack(anchor="center")
    card_preview.pack_propagate(False)

    # Chip image (represented as a small rectangle)
    chip_frame = ctk.CTkFrame(
        card_preview,
        width=50,
        height=40,
        fg_color="#FFD700",
        corner_radius=5
    )
    chip_frame.place(x=30, y=50)

    # Card type label (top right)
    card_type_preview = ctk.CTkLabel(
        card_preview,
        text="VISA",
        font=("Helvetica", 20, "bold"),
        text_color="white"
    )
    card_type_preview.place(x=270, y=20)

    # Card number preview
    card_number_preview = ctk.CTkLabel(
        card_preview,
        text="**** **** **** ****",
        font=("Courier", 24, "bold"),
        text_color="white"
    )
    card_number_preview.place(x=30, y=100)

    # Expiry date preview
    expiry_label = ctk.CTkLabel(
        card_preview,
        text="VALID\nTHRU",
        font=("Helvetica", 8),
        text_color="white"
    )
    expiry_label.place(x=30, y=140)

    expiry_preview = ctk.CTkLabel(
        card_preview,
        text="MM/YY",
        font=("Helvetica", 14, "bold"),
        text_color="white"
    )
    expiry_preview.place(x=30, y=165)

    # Cardholder name preview
    cardholder_preview = ctk.CTkLabel(
        card_preview,
        text="CARDHOLDER NAME",
        font=("Helvetica", 14, "bold"),
        text_color="white"
    )
    cardholder_preview.place(x=150, y=165)

    def validate_card_number(event=None):
        value = card_entry.get().replace(" ", "")
        if not value.isdigit():
            card_entry.delete(0, "end")
            card_entry.insert(0, "".join(c for c in value if c.isdigit()))
        if len(value) > 16:
            card_entry.delete(16, "end")

    def validate_month(event=None):
        value = month_entry.get()
        if not value.isdigit():
            month_entry.delete(0, "end")
            return
        if len(value) > 2:
            month_entry.delete(2, "end")
        if value and int(value) > 12:
            month_entry.delete(0, "end")
            month_entry.insert(0, "12")

    def validate_year(event=None):
        value = year_entry.get()
        if not value.isdigit():
            year_entry.delete(0, "end")
            return
        if len(value) > 2:
            year_entry.delete(2, "end")
        # Only validate the range if we have 2 digits
        if len(value) == 2 and (int(value) < 25 or int(value) > 99):
            messagebox.showerror("Error", "Please input the correct year")
            year_entry.delete(0, "end")

    def validate_cvv(event=None):
        value = cvv_entry.get()
        if not value.isdigit():
            cvv_entry.delete(0, "end")
            cvv_entry.insert(0, "".join(c for c in value if c.isdigit()))
        if len(value) > 3:
            cvv_entry.delete(3, "end")

    def update_card_preview(event=None):
        # Update card number
        card_num = card_entry.get().strip()
        if card_num:
            # Format card number with spaces and mask first 12 digits
            masked_num = '**** **** **** ' + card_num[-4:] if len(card_num) >= 4 else card_num
            card_number_preview.configure(text=masked_num)
        else:
            card_number_preview.configure(text="**** **** **** ****")

        # Update expiry date
        month = month_entry.get().strip()
        year = year_entry.get().strip()
        if month or year:
            expiry_preview.configure(text=f"{month.zfill(2)}/{year.zfill(2)}")
        else:
            expiry_preview.configure(text="MM/YY")

        # Update card type and color
        card_type = card_type_var.get().upper()
        card_type_preview.configure(text=card_type)
        
        # Update card color based on type
        if card_type == "VISA":
            card_preview.configure(fg_color="#0066cc")
        elif card_type == "MASTERCARD":
            card_preview.configure(fg_color="#EB001B")
        elif card_type == "PAYPAL":
            card_preview.configure(fg_color="#003087")
            
        # If we're in edit mode, update the saved card preview as well
        if hasattr(frame, 'current_payment_id'):
            temp_payment = {
                'payment_method': card_type_var.get(),
                'card_number': card_num,
                'expiry_date': f"{month.zfill(2)}/{year.zfill(2)}" if month and year else "MM/YY"
            }
            create_payment_card(temp_payment)

    # Bind validation functions
    card_entry.bind("<KeyRelease>", validate_card_number)
    month_entry.bind("<KeyRelease>", validate_month)
    year_entry.bind("<KeyRelease>", validate_year)
    cvv_entry.bind("<KeyRelease>", validate_cvv)

    # Bind the update function to all relevant widgets
    card_entry.bind("<KeyRelease>", update_card_preview)
    month_entry.bind("<KeyRelease>", update_card_preview)
    year_entry.bind("<KeyRelease>", update_card_preview)
    card_type_dropdown.bind("<KeyRelease>", update_card_preview)

    def save_payment():
        """Save or update payment method"""
        nonlocal saved_card_frame
        
        # Validate all fields
        card_num = card_entry.get().strip()
        month = month_entry.get().strip()
        year = year_entry.get().strip()
        cvv = cvv_entry.get().strip()
        card_type = card_type_var.get().lower()

        if not all([card_num, month, year, cvv]):
            messagebox.showerror("Error", "Please fill in all fields")
            return

        if len(card_num) < 12:
            messagebox.showerror("Error", "Invalid card number")
            return

        if not (1 <= int(month) <= 12):
            messagebox.showerror("Error", "Invalid month")
            return

        if not (25 <= int(year) <= 99):
            messagebox.showerror("Error", "Invalid year")
            return

        if len(cvv) != 3:
            messagebox.showerror("Error", "Invalid security code")
            return

        # Create payload
        payload = {
            "payment_method": card_type,
            "card_number": card_num,
            "expiry_date": f"{month}/{year}",
            "cvv": cvv
        }

        try:
            headers = {"Authorization": f"Bearer {frame.token}"}
            
            if hasattr(frame, 'current_payment_id'):
                # Update existing payment
                response = requests.put(
                    f"{API_URL}/payment/{frame.current_payment_id}",
                    headers=headers,
                    json=payload
                )
                action = "updated"
            else:
                # Add new payment
                response = requests.post(
                    f"{API_URL}/payment/add",
                    headers=headers,
                    json=payload
                )
                action = "added"

            if response.status_code == 200:
                messagebox.showinfo("Success", f"Payment method {action} successfully!")
                # Clear form and reset state
                clear_form()
                # Remove the current_payment_id if it exists
                if hasattr(frame, 'current_payment_id'):
                    delattr(frame, 'current_payment_id')
                # Refresh to show updated data
                refresh_payments()
                # Disable form inputs after successful save
                disable_form_inputs()
            else:
                error_msg = response.json().get("detail", f"Failed to {action} payment method")
                messagebox.showerror("Error", error_msg)
        except Exception as e:
            messagebox.showerror("Error", f"Failed to save payment method: {str(e)}")

    # Save Button
    save_button = ctk.CTkButton(
        form_frame,
        text="Save Payment Method",
        command=save_payment,
        height=45,
        corner_radius=8,
        font=("Helvetica", 14, "bold"),
        fg_color=SHOPPING,
        hover_color="#0096ff",
    )
    save_button.pack(fill="x", pady=(30, 0))

    # Initial refresh of payment methods
    refresh_payments()

    return frame
